source("start.R")

sink("log/FA.log")

for (i in 1:length(dataset.names)) {
  dataset.name <- dataset.names[i]

  cat("Starting FA for dataset \"", dataset.name, "\".\n", sep="")

  dataset <- get.extended.dataset(dataset.name)
  p <- ncol(dataset)
  
  # Max-factors:
  factors <- seq(1, floor(1/2+p-(1/2)*sqrt(1+8*p)), 1)

  for (f in factors) {
    cat("Factors =", f, "\n")
    
    fit <- factanal(dataset, f)

    if (fit$dof == 0) {
      cat("Degrees of freedom = 0, skipping.\n")
    } else if (fit$PVAL < 0.05) {  
      cat("P-value =", fit$PVAL, "< 0.05, skipping.\n")
    } else {
      print(fit, digits=2, cutoff=0.3, sort=TRUE)
      load <- fit$loadings 

      pdf(paste("figures/FA-", dataset.name, "-", f, ".pdf", sep=""))
      plot(load, 
        type="n", 
        main=paste("Factor analysis of ", dataset.name, " with ", f, " factors", sep=""), 
        xlab="Factor 1", 
        ylab="Factor 2") # plot factor 1 by 2
        
      #text(load, labels=names(dataset), cex=.7)
      text(load, labels=names(dataset))
      dev.off()
    }
  }
  
  cat("Dataset \"", dataset.name, "\" done.\n\n", sep="")
}

sink()

