source("start.R")

sink("log/PCA.log")

for (i in 1:length(dataset.names)) {
  dataset.name <- dataset.names[i]

  cat("Starting PCA for dataset \"", dataset.name, "\".\n", sep="")

  dataset <- get.extended.dataset(dataset.name)
  p <- ncol(dataset)
  
  fit <- princomp(dataset, cor=TRUE)
  print(summary(fit))
  print(loadings(fit))
  
  pdf(paste("figures/PCA-", dataset.name, "-screeplot.pdf", sep=""))
  variance.percentage <- 100*fit$sdev^2 / sum(fit$sdev^2)
  plot(variance.percentage, 
    main=paste("Principal component analysis of ", dataset.name, " - scree plot", sep=""), 
    xlab="Component", ylab="Percentage of total variance", type="b")
  text(variance.percentage, adj=c(0, -0.5), labels=paste(round(variance.percentage, 2), "%"), cex=0.8)
  dev.off()

  if (FALSE) {
    pdf(paste("figures/PCA-", dataset.name, "-biplot.pdf", sep=""))
    biplot(fit,
      main=paste("Principal component analysis of ", dataset.name, " - biplot", sep=""),
      xlab="SPC1", ylab="SPC2",
      cex=0.7)
    dev.off()
  }

  if (FALSE) {
    pdf(paste("figures/PCA-", dataset.name, "-biplot-groups.pdf", sep=""))
    cluster <- kmeans(fit$scores[, 1:2], 3)$cluster
    cols <- c("red", "blue", "green")
    biplot(fit,
      col=cols[cluster],
      main=paste("Principal component analysis of ", dataset.name, " - biplot", sep=""),
      cex=0.7)
    legend("topright", c("Group 1", "Group 2", "Group 3"), col=cols)
    dev.off()
  }

  pdf(paste("figures/PCA-", dataset.name, "-SPC1-vs-SPC2.pdf", sep=""))
  plot(fit$scores[, 1], fit$scores[, 2], 
    main=paste("Principal component analysis of ", dataset.name, sep=""), 
    xlab="SPC1", ylab="SPC2")
  dev.off()
  
  cat("Dataset \"", dataset.name, "\" done.\n\n", sep="")
}

sink()

