library(xtable)

source("start.R")
source("include-ancestor.R")
source("include-marginal-logic.R")

log.name.general <- "log/ancestor.log"
SUPRESS_LOG <- FALSE
prob.matrix.decimals <- 3

ancestor.set.limits <- c(4, 5, 6)
ancestor.set.thresholds <- c(0.1, 0.15, 0.2)
ancestor.set.mutations <- c(TRUE, FALSE)

#ancestor.set.limits <- c(4)
#ancestor.set.thresholds <- c()
#ancestor.set.mutations <- c(FALSE)

ancestor.set.limits.length <- length(ancestor.set.limits)
ancestor.set.thresholds.length <- length(ancestor.set.thresholds)
ancestor.set.mutations.length <- length(ancestor.set.mutations)

ancestor.set.limits <- c(ancestor.set.limits, rep(NA, ancestor.set.thresholds.length))
ancestor.set.thresholds <- c(rep(NA, ancestor.set.limits.length), ancestor.set.thresholds)
ancestor.set.stop.length <- length(ancestor.set.limits)

# To clear log
sink(log.name.general)
sink()

unobserved.prob.matrix <- matrix(NA, nrow=ancestor.set.stop.length*ancestor.set.mutations.length, ncol=length(dataset.names)+2, dimnames=list(c(), c("Stop criteria", "Mutation", dataset.names)))

deviance.relative.predicted.matrix <- matrix(NA, 
  nrow=ancestor.set.stop.length*ancestor.set.mutations.length, 
  ncol=length(dataset.names)+2,
  dimnames=list(c(), c("Stop criteria", "Mutation", paste("¤DATASET:", dataset.names, "¤", sep=""))))

for (i in 1:length(dataset.names)) {
#for (i in 1:1) {
  dataset.name <- dataset.names[i]

  cat("Starting ancestral awareness for dataset \"", dataset.name, "\".\n", sep="")
  
  sink(log.name.general, append = TRUE)
  cat("Starting ancestral awareness for dataset \"", dataset.name, "\".\n\n", sep="")
  sink()

  haplotypes <- get.compact.dataset(dataset.name)
  haplotypes.count <- get.haplotypes.count(dataset.name)
  
  dataset <- get.extended.dataset(dataset.name)
  observation.count <- get.observation.count(dataset.name)
  loci.count <- get.loci.count(dataset.name)
  loci <- 1:loci.count
  loci.names <- colnames(dataset)
  loci.levels <- get.loci.levels(dataset.name)
  
  for (j in 1:ancestor.set.stop.length) {  
    ancestor.set.limit <- ancestor.set.limits[j]
    ancestor.set.threshold <- ancestor.set.thresholds[j]

    log.name.base <- character(0)

    if (is.na(ancestor.set.limit)) {
      ancestor.set.limit <- NULL
      log.name.base <- paste("threshold-", ancestor.set.threshold, ".log", sep="")
    } else {
      # Only two possibilities
      if (!is.na(ancestor.set.threshold)) stop("!is.na(ancestor.set.threshold)")
      ancestor.set.threshold <- NULL
      log.name.base <- paste("limit-", ancestor.set.limit, ".log", sep="")
    }    
  
    for (m in 1:ancestor.set.mutations.length) {
      mutation <- ancestor.set.mutations[m]
      
      if (mutation) {
        log.name <- paste("log/ancestor/", dataset.name, "-mutation-", log.name.base, sep="")
      } else {
        log.name <- paste("log/ancestor/", dataset.name, "-", log.name.base, sep="")
      }

      sink(log.name.general, append = TRUE)      
      cat("Limit                    =", 
        ifelse(is.null(ancestor.set.limit), "NULL", ancestor.set.limit), "\n")
      cat("Threshold                =", 
        ifelse(is.null(ancestor.set.threshold), "NULL", ancestor.set.threshold), "\n")
      cat("Mutation                 =", ifelse(mutation, "Yes", "No"), "\n")
      sink()

      sink(log.name)
      cat("Limit                    =", 
        ifelse(is.null(ancestor.set.limit), "NULL", ancestor.set.limit), "\n")
      cat("Threshold                =", 
        ifelse(is.null(ancestor.set.threshold), "NULL", ancestor.set.threshold), "\n")
      cat("Mutation                 =", ifelse(mutation, "Yes", "No"), "\n")
      
      prob.sum <- 0
      probs <- numeric(haplotypes.count)
      
      for (k in 1:haplotypes.count) {
        h <- as.numeric(haplotypes[k, loci])
      
        prob <- 0
        
        if (mutation) {
          prob <- calc.approx.probability.with.mutation(h, ancestor.limit = ancestor.set.limit, observations.left.threshold = ancestor.set.threshold)
        } else {
          prob <- calc.approx.probability(h, ancestor.limit = ancestor.set.limit, observations.left.threshold = ancestor.set.threshold)
        }

        probs[k] <- prob
        prob.sum <- prob.sum + prob
        
        sink()
        cat(log.name, ": ", round(100*k / haplotypes.count, 2), "%\n", sep="")
        sink(log.name, append = TRUE)
      }

      row.number <- (m-1)*ancestor.set.stop.length + j

      # This is written each time a dataset is processed, but what the heck
      unobserved.prob.matrix[row.number, 1] <- ifelse(is.null(ancestor.set.limit), 
        paste(ancestor.set.threshold, "threshold"), 
        paste(ancestor.set.limit, "ancestors"))
      unobserved.prob.matrix[row.number, 2] <- ifelse(mutation, "Yes", "No")

      deviance.relative.predicted.matrix[row.number, 1] <- ifelse(is.null(ancestor.set.limit), 
        paste(ancestor.set.threshold, "threshold"), 
        paste(ancestor.set.limit, "ancestors"))
      deviance.relative.predicted.matrix[row.number, 2] <- ifelse(mutation, "Yes", "No")
      
      # This, on the other hand, is not...
      unobserved.prob.matrix[row.number, i+2] <- round(1-prob.sum, prob.matrix.decimals)

      deviance.relative.predicted.matrix[row.number, i+2] <- round(deviance.relfreq.predicted(haplotypes$N, probs), prob.matrix.decimals)
     
      cat("\n\n**************************\n\n\n")
      cat("Covered probability mass   =", round(prob.sum, prob.matrix.decimals), "\n")
      cat("Uncovered probability mass =", unobserved.prob.matrix[row.number, i+2], "\n")
        
      sink() # sink(log.name)

      sink(log.name.general, append = TRUE)
      cat("Uncovered probability mass =", unobserved.prob.matrix[row.number, i+2], "\n")
      cat("Deviance rel. vs. pred.    =", deviance.relative.predicted.matrix[row.number, i+2], "\n\n")
      sink()
    } # for (m in 1:ancestor.set.mutations.length)
  }

  cat("Dataset \"", dataset.name, "\" done.\n\n", sep="")

  sink(log.name.general, append = TRUE)
  cat("Dataset \"", dataset.name, "\" done.\n\n", sep="")
  sink()
}

#sink()

tableOutput <- xtable(unobserved.prob.matrix,
      align=c("r", "l", "l", rep("r", length(dataset.names))),
      hline.after=seq(-1, ancestor.set.stop.length*ancestor.set.mutations.length, 1),
      label=paste("tab:ancestor-unobserved-prob-matrix", sep=""), 
      caption=paste("Table of unobserved probability mass using ancestral awareness. Refer to table \\ref{tab:datasets-unobserved-prob-matrix}."))
#      caption=paste("Table of unobserved probability mass using ancestral awareness. \\dataBerlin{} estimated to $0.364$ (and with $95\%$ probability within $[ 0.321 ; 0.408 ]$), \\dataDane{} estimated to $0.602$ (and within $95\\%$ probability within $[ 0.510 ; 0.694 ]$), and \\dataSomali{} estimated to $0.277$ (and within $95\\%$ probability within $[ 0.212 ; 0.342 ]$). Also refer to table \\ref{tab:datasets-unobserved-prob-matrix}.")) 
print(tableOutput, file=paste("out/ancestor-unobserved-probability-mass.tex", sep=""), include.rownames = FALSE)


#deviance.relative.predicted.matrix <- t(deviance.relative.predicted.matrix)
tableOutput <- xtable(deviance.relative.predicted.matrix,
      align=c("l", rep("r", ncol(deviance.relative.predicted.matrix))),
      label=paste("tab:ancestor-deviance-relative-predicted", sep=""), 
      caption=paste("Comparing deviance between relative frequencies and predicted probabilities found using ancestral awareness."))
filename <- paste("out/ancestor-deviance-relative-predicted.tex", sep="")
print(tableOutput, file=filename)
do.replacements(filename)

