log.filename <- "log/classification-evidence.log"

source("start.R")
source("include-classification-model-specification.R")
source("include-classification.R")
source("include-classification-evidence-2-contributors.R")
source("include-marginal-logic.R")

example.result <- evidence.two.contributors(dataset.name = "dane", 
  classification.model.number = 1, 
  verbose = TRUE,
  keep.contributors.from.last.run = TRUE)

if (F) {
  K <- 10000
  simulation.results <- vector("list", K)

  for (i in 1:K) {
    # classification.model.number = NULL to use model.fits from prior usage
    
    simulation.results[[i]] <- evidence.two.contributors(dataset.name = "dane", 
      classification.model.number = NULL, 
      verbose = FALSE,
      keep.contributors.from.last.run = FALSE)

    cat(round(100*i/K, 3), "%\n")
  }

  contributors.places <- sapply(simulation.results, function(x) x$contributors.place)
  haplotype.universe.sizes <- sapply(simulation.results, function(x) x$haplotype.universe.size)
  prob.sums <- sapply(simulation.results, function(x) x$prob.sum)
  prob.prods <- sapply(simulation.results, function(x) prod(x$contributors.probs))
  prob.prods.proportion <- prob.prods / prob.sums

  #cor.test(contributors.places, haplotype.universe.sizes)
  #cor.test(contributors.places, prob.sums)
  #cor.test(haplotype.universe.sizes, prob.sums)

  cat("Mean place =", mean(contributors.places), "\n")
  cat("SD places =", sd(contributors.places), "\n")
  cat("quantile(contributors.places, probs = c(0.025, 0.975)) =", quantile(contributors.places, probs = c(0.025, 0.975)), "\n")
  cat("mean +/- 1.96*SD = ", mean(contributors.places) + c(-1, 1) * 1.96 * sd(contributors.places), "\n")
  
  pdf("figures/classification-evidence-2-contributors-rank-hist.pdf")
  hist(contributors.places, main = "Histogram of true contributors' rank", xlab = "True contributors' rank")
  dev.off()
  
  pdf("figures/classification-evidence-2-contributors-rank-boxplot.pdf")
  boxplot(contributors.places, main = "Histogram of true contributors' rank", xlab = "True contributors' rank")
  dev.off()
  
  pdf("figures/classification-evidence-2-contributors-rank-boxplot.pdf")
  rank.boxplot <- boxplot(contributors.places, main = "Histogram of true contributors' rank", xlab = "True contributors' rank")
  dev.off()
  
  rank.outliers <- table(rank.boxplot$out)
  haplotype.universe.sizes.outliers <- table(haplotype.universe.sizes[which(contributors.places > 10)])
  
  cat("Table of true contributors rank outliers:\n")
  print(rank.outliers)
  
  cat("In percent out of", K, "simulations:\n")
  print(rank.outliers / K)
  
  cat("Table of sizes of modified haplotype universe for simulations where true contributors has rank > 10:\n")
  print(haplotype.universe.sizes.outliers)
  
  cat("In percent out of", K, "simulations:\n")
  print(round(prop.table(haplotype.universe.sizes.outliers), 3))
  
  cat("Number of simulations causing rank > 10 =", length(which(contributors.places > 10)), "=", length(which(contributors.places > 10)) / K, "\n")
  
  for (place in c(10, 5:1)) {
    place.count <- length(which(contributors.places <= place))
    cat("Number of simulations causing rank <=", place, "=", place.count, 
      "=", place.count / K, "\n")
  }
  
  #places.max <- 100
  places.max <- 20
  percentages <- places.max
  
  for (place in 1:places.max) {
    percentages[place] <- length(which(contributors.places <= place)) / K
  }
  
  pdf("figures/classification-evidence-2-contributors-rank-top-cumulative.pdf")
  plot(1:places.max, percentages, main = "Cumulative probability", xlab = "Top-x", ylab = "Probability that the true contributors' are in the top-x", xlim=c(1, places.max+1), ylim=c(0.6, 1.02))
  text(1:places.max, percentages, labels = round(percentages, 4), srt= 90, pos = 1, offset = 1.8)
  text(1:places.max, percentages, labels = 1:places.max, pos = 3)
  dev.off()
}

