get.compact.dataset <- function(dataset.name) {
  return(read.csv2(paste("data/data-compact-", dataset.name, ".csv", sep=""), row.names=1))
}

get.extended.dataset <- function(dataset.name) {
  return(read.csv2(paste("data/data-extended-", dataset.name, ".csv", sep=""), row.names=1))
}

# In theory it is stupid to load the dataset this often, 
# but it is really quick and it makes the code more safe
# and easy to include or exclude datasets.

get.loci.count <- function(dataset.name) {
  extended.dataset <- get.extended.dataset(dataset.name)  
  return(ncol(extended.dataset))
}

get.loci.names <- function(dataset.name) {
  extended.dataset <- get.extended.dataset(dataset.name)  
  return(colnames(extended.dataset))
}

get.loci.levels <- function(dataset.name) {
  loci.count <- get.loci.count(dataset.name)
  dataset <- get.compact.dataset(dataset.name)
  
  loci.levels <- vector("list", loci.count)
  for (l in 1:loci.count) {
    loci.levels[[l]] <- sort(unique(dataset[, l]))
  }

  return(loci.levels)
}

get.observation.count <- function(dataset.name) {
  extended.dataset <- get.extended.dataset(dataset.name)  
  return(nrow(extended.dataset))
}

get.haplotypes.count <- function(dataset.name) {
  compact.dataset <- get.compact.dataset(dataset.name)
  return(nrow(compact.dataset))
}

# 1: Singletons
# 2: Doubletons
#    ect.
get.Mk.count <- function(dataset.name, k) {
  if (k < 1) {
    stop("k < 1 not allowed")
  }

  k <- as.character(k)

  compact.dataset <- get.compact.dataset(dataset.name)
  
  # The following would suffice if we could guarantee that the dataset
  # contained singletons:
  # return(as.numeric(table(compact.dataset$N)[1]))
  # But we cannot, so:
  compact.dataset.matrix <- as.matrix(table(compact.dataset$N))
  if (is.null(compact.dataset$N)) stop("compact.dataset$N == NULL")
  
  row.number <- which(rownames(compact.dataset.matrix) == k)

  # No k-tons was in the database
  if (length(row.number) == 0) {
    return(0)
  } else if (length(row.number) > 1) {
    stop("The k =", k, "resulted in several rows.")
  }

  return(as.numeric(compact.dataset.matrix[row.number, 1]))
}

get.singletons.count <- function(dataset.name) {
  return(get.Mk.count(dataset.name, 1))
}

get.doubletons.count <- function(dataset.name) {
  return(get.Mk.count(dataset.name, 2))
}

get.haplotype.count.table <- function(dataset.name) {
  compact.dataset <- get.compact.dataset(dataset.name)
  if (is.null(compact.dataset$N)) stop("compact.dataset$N == NULL")
  return(table(compact.dataset$N))
}

get.correlation.matrix <- function(dataset.name, exact.p.value, ...) {
  extended.dataset <- get.extended.dataset(dataset.name)
  M <- ncol(extended.dataset)
  cnames <- colnames(extended.dataset)
  result.matrix <- cor(extended.dataset, ...)  
  
  for (i in 1:(M-1)) {
    for (j in (i+1):M) {
      if (exact.p.value) {
        result.matrix[i, j] <- cor.test(extended.dataset[, i], extended.dataset[, j], ...)$p.value
      } else {
        result.matrix[i, j] <- cor.test(extended.dataset[, i], extended.dataset[, j], exact = FALSE, ...)$p.value
      }
    }
  }
  
  for (i in 1:M) {
    result.matrix[i, i] <- NA
  }
  
  return(result.matrix)
}

matrix.as.latex <- function(dataset.name) {
  extended.dataset <- get.extended.dataset(dataset.name)  
  return(cor(extended.dataset))
}

