do.replacements <- function(filename) {
  for (i in 1:length(dataset.names)) {
    dataset.name <- dataset.names[i]  
    dataset.name.cc <- dataset.name
    substring(dataset.name.cc, 0, 1) <- toupper(substr(dataset.name, 0, 1))
    
    system(paste("sed -i 's/¤DATASET:", dataset.name, "¤/\\\\data", dataset.name.cc, "{}/g' ", filename, sep=""))

    loci.names <- get.loci.names(dataset.name)

    if (exists("loci.names")) {
      for (locus.name in loci.names) {
        system(paste("sed -i 's/¤LOCUS:", locus.name, "¤/\\\\texttt{", locus.name, "}/g' ", filename, sep=""))
      }
    }
  }  

  if (exists("classification.model.names")) {
    for (classification.model.name in classification.model.names) {
      system(paste("sed -i 's/", classification.model.name, "/\\\\texttt{", classification.model.name, "}/g' ", filename, sep=""))
    }
  }
}

# locus.to.ignore: No need to compare marginals for locus.to.ignore locus -- it is
# by definition equal to the observed
analyze.single.marginals <- function(dataset.true.model, dataset.to.compare, 
  true.model.name, compare.name, locus.to.ignore = NULL) {

  dataset.true.model <- as.data.frame(dataset.true.model)
  dataset.to.compare <- as.data.frame(dataset.to.compare)

  use.N.column <- FALSE

  # When comparing simulated with predicted marginals, this is occuring because
  # the N's are not integers hence we cannot simply insert N_i rows, but have to 
  # use xtabs(dataset.to.compare$N ~ dataset.to.compare[, locus])
  if (is.null(dataset.to.compare$N)) {
    if (ncol(dataset.true.model) != ncol(dataset.to.compare)) {
      stop("ncol(dataset.true.model) != ncol(dataset.to.compare)")
    }
    
    if (any(colnames(dataset.true.model) != colnames(dataset.to.compare))) {
      stop("colnames(dataset.true.model) != colnames(dataset.to.compare)")
    }
  } else {
    use.N.column <- TRUE

    if (ncol(dataset.true.model) != (ncol(dataset.to.compare) - 1)) {
      stop("ncol(dataset.true.model) != (ncol(dataset.to.compare) - 1) (because of the N-column)")
    }

    # Could be done nicer...
    dataset.to.compare.colnames <- colnames(dataset.to.compare)
    dataset.to.compare.colnames <- dataset.to.compare.colnames[-which(dataset.to.compare.colnames == "N")]
    
    if (any(colnames(dataset.true.model) != dataset.to.compare.colnames)) {
      stop("colnames(dataset.true.model) != colnames(dataset.to.compare)")
    }
  }
  
  cat("=====================\n")
  cat("Single marginals:\n")
  cat("=====================\n")

  #loci.used <- colnames(dataset.true.model)
  loci.used <- loci

  if (!is.null(locus.to.ignore)) {
    loci.used <- loci.used[-locus.to.ignore]
  }

  loci.used.count <- length(loci.used)
  
  deviances <- numeric(loci.used.count)
  p.values <- numeric(loci.used.count)
  loci.rejected <- 0
  
  for (i in 1:loci.used.count) {
    locus <- loci.used[i]   

    single.marginals.true <- as.matrix(table(dataset.true.model[, locus]))    
    single.marginals.compare <- NULL

    # Refer to note at the declaration of the variable
    if (use.N.column) {
      single.marginals.compare <- prop.table(xtabs(dataset.to.compare$N ~ dataset.to.compare[, locus]))
    } else {
      single.marginals.compare <- prop.table(table(dataset.to.compare[, locus]))
    }

    probs.true <- prop.table(single.marginals.true)
    probs.compare <- as.matrix(single.marginals.compare)

    cat("---------------------\n")
    cat(loci.names[locus], "\n")
    cat("---------------------\n")
    cat(true.model.name, ":\n")
    print(probs.true)

    cat(compare.name, ":\n")
    print(probs.compare)

    if (any(rownames(probs.true) != rownames(probs.compare))) stop("rownames(probs.true) != rownames(probs.compare)")

    # na.rm = TRUE because log(1) = 0, so NA's can just be removed
    d <- -2 * sum(single.marginals.true * log(probs.compare / probs.true), na.rm = TRUE)
    cat("Deviance for ", loci.names[locus], " = ", round(d, 3), "\n", sep="")

    p.value <- 1-pchisq(d, nrow(probs.true)-1)
    cat("One tailed p-value = ", p.value, "\n")

    if (p.value < 0.05) {
      cat("We !!! ___REJECT___ !!! that the model to compare with explains the true model/observations for this locus.\n")
      loci.rejected <- loci.rejected + 1
    } else {
      cat("We cannot reject that the model to compare with explains the true model/observations for this locus.\n")
    }

    deviances[i] <- d
    p.values[i] <- p.value
  }

  deviance.sum <- sum(deviances)
  
  cat("Sum of deviances = ", round(deviance.sum, 3), "\n", sep="")
  return(list(deviance.sum = deviance.sum, deviances = deviances, p.values = p.values, loci.rejected = loci.rejected))
}


# locus.to.ignore: No need to compare marginals for locus.to.ignore locus -- it is
# by definition equal to the observed
analyze.pairwise.marginals <- function(dataset.true.model, dataset.to.compare, 
  true.model.name, compare.name, locus.to.ignore = NULL) {

  dataset.true.model <- as.data.frame(dataset.true.model)
  dataset.to.compare <- as.data.frame(dataset.to.compare)

  use.N.column <- FALSE

  # When comparing simulated with predicted marginals, this is occuring because
  # the N's are not integers hence we cannot simply insert N_i rows, but have to 
  # use xtabs(dataset.to.compare$N ~ dataset.to.compare[, locus])
  if (is.null(dataset.to.compare$N)) {
    if (ncol(dataset.true.model) != ncol(dataset.to.compare)) {
      stop("ncol(dataset.true.model) != ncol(dataset.to.compare)")
    }
    
    if (any(colnames(dataset.true.model) != colnames(dataset.to.compare))) {
      stop("colnames(dataset.true.model) != colnames(dataset.to.compare)")
    }
  } else {
    use.N.column <- TRUE

    if (ncol(dataset.true.model) != (ncol(dataset.to.compare) - 1)) {
      stop("ncol(dataset.true.model) != (ncol(dataset.to.compare) - 1) (because of the N-column)")
    }

    # Could be done nicer...
    dataset.to.compare.colnames <- colnames(dataset.to.compare)
    dataset.to.compare.colnames <- dataset.to.compare.colnames[-which(dataset.to.compare.colnames == "N")]
    
    if (any(colnames(dataset.true.model) != dataset.to.compare.colnames)) {
      stop("colnames(dataset.true.model) != colnames(dataset.to.compare)")
    }
  }
  
  cat("=====================\n")
  cat("Pairwise marginals:\n")
  cat("=====================\n")

  #loci.used <- colnames(dataset.true.model)
  loci.used <- loci

  if (!is.null(locus.to.ignore)) {
    loci.used <- loci.used[-locus.to.ignore]
  }

  loci.used.count <- length(loci.used)

  combinations.count <- choose(loci.used.count, 2)
  deviances <- numeric(combinations.count)
  p.values <- numeric(combinations.count)
  comparison.labels <- numeric(combinations.count)
  index <- 1
  loci.rejected <- 0
  
  for (i in 1:(loci.used.count-1)) {
    locus1 <- loci.used[i]

    for (j in (i+1):loci.used.count) {
      locus2 <- loci.used[j]
      
      pairwise.marginals.true <- as.matrix(table(dataset.true.model[, c(locus1, locus2)]))    
      pairwise.marginals.compare <- NULL

      # Refer to note at the declaration of the variable
      if (use.N.column) {
        pairwise.marginals.compare <- prop.table(xtabs(dataset.to.compare$N ~ dataset.to.compare[, locus1] + dataset.to.compare[, locus2]))
      } else {
        pairwise.marginals.compare <- prop.table(table(dataset.to.compare[, c(locus1, locus2)]))
      }

      probs.true <- prop.table(pairwise.marginals.true)
      probs.compare <- as.matrix(pairwise.marginals.compare)

      cat("---------------------\n")
      cat(loci.names[locus1], "vs.", loci.names[locus2], "\n")
      cat("---------------------\n")
      cat(true.model.name, ":\n")
      print(probs.true)

      cat(compare.name, ":\n")
      print(probs.compare)

      if (any(rownames(probs.true) != rownames(probs.compare))) stop("rownames(probs.true) != rownames(probs.compare)")

      # na.rm = TRUE because log(1) = 0, so NA's can just be removed
      d <- -2 * sum(pairwise.marginals.true * log(probs.compare / probs.true), na.rm = TRUE)
      cat("Deviance for ", loci.names[locus1], " vs. ", loci.names[locus2], " = ", round(d, 3), "\n", sep="")

      p.value <- 1-pchisq(d, nrow(probs.true)-1)
      cat("One tailed p-value = ", p.value, "\n")

      if (p.value < 0.05) {
        cat("We !!! ___REJECT___ !!! that the model to compare with explains the true model/observations for this locus.\n")
        loci.rejected <- loci.rejected + 1
      } else {
        cat("We cannot reject that the model to compare with explains the true model/observations for this locus.\n")
      }

      deviances[index] <- d
      p.values[index] <- p.value
      comparison.labels <- paste(loci.names[locus1], "vs.", loci.names[locus2])
      index <- index + 1
    }
  }

  deviance.sum <- sum(deviances)
  
  cat("Sum of deviances = ", round(deviance.sum, 3), "\n", sep="")
  return(list(deviance.sum = deviance.sum, deviances = deviances, p.values = p.values, loci.rejected = loci.rejected, comparison.labels = comparison.labels))
}

create.perdataset.output.single.marginals <- function(method.name, matrix.with.summaries, dataset.name, matrix.description, true.model.name, compare.name) {
  matrix.description.lower <- tolower(matrix.description)
  identifier <- paste("single-marginals-perdataset-", dataset.name, "-", matrix.description.lower, "-", true.model.name, "-vs-", compare.name, sep="")

  #matrix.with.summaries <- t(matrix.with.summaries)

  # pretty printing ->
  display.vector <- c("s", rep("f", ncol(matrix.with.summaries)))

  # Because of transpose, we take [,1] which is rows instead of [,2] which is columns
  #colums.with.small.values <- unique(as.vector(which(matrix.with.summaries < 0.001 & matrix.with.summaries > 0, arr.ind = TRUE)[, 1]))
  colums.with.small.values <- unique(as.vector(which(matrix.with.summaries < 0.001 & matrix.with.summaries > 0, arr.ind = TRUE)[, 2]))

  if (length(colums.with.small.values) > 0) {
    display.vector[colums.with.small.values + 1] <- "e"
  }
  # <- pretty printing

  capt <- paste(matrix.description, " for each loci for ¤DATASET:", dataset.name, "¤ dataset for ", true.model.name, " single marginals vs. ", compare.name, " single marginals for the ", method.name, " method specified in each row.", sep="")

  if (any(is.na(matrix.with.summaries))) {
    capt <- paste(capt, " The missing value is for the locus where the observed marginal distribution has been used.", sep="")
  }

  tableOutput <- xtable(matrix.with.summaries,
        digits=3,
        display=display.vector,
        align=c("l", rep("r", ncol(matrix.with.summaries))),
        label=paste("tab:", identifier, sep=""), 
        caption=capt)
  filename <- paste("out/", method.name, "/", identifier, ".tex", sep="")
  print(tableOutput, file=filename)
  do.replacements(filename)

  # Include table in LaTeX automatically
  sink() # stop log
  sink(latex.filename, append = TRUE)
  cat("\\input{", identifier, "}\n\\clearpage\n", sep="")
  sink()
  sink(log.filename, append = TRUE) # start log again
      
  #pdf(paste("figures/", method.name, "/", identifier, ".pdf", sep=""))
  #org.par <- par("mar")
  #par(mar = c(7.1, 4.1, 4.1, 2.1))
  #boxplot(matrix.with.summaries, 
  #  main=paste(matrix.description, "on each loci for", dataset.name), ylab=matrix.description, las = 2) 
  #par(mar = org.par)
  #dev.off()

  #pdf(paste("figures/", method.name, "/", identifier, "-without-svms.pdf", sep=""))
  #org.par <- par("mar")
  #par(mar = c(7.1, 4.1, 4.1, 2.1))
  #boxplot(matrix.with.summaries[,-which(substr(classification.model.names, 0, 3) == "svm")], 
  #  main=paste(matrix.description, "on each loci for", dataset.name, "excluding SVM-methods"), ylab=matrix.description, las = 2) 
  #par(mar = org.par)
  #dev.off()  
}

create.overall.output.marginals <- function(method.name, marginals.type, matrix.with.summaries, matrix.description.short, matrix.description.long, true.model.name, compare.name, columns.is.int = FALSE) {
  identifier <- paste(marginals.type, "-marginals-overall-", matrix.description.short, "-", true.model.name, "-vs-", compare.name, sep="")

  matrix.with.summaries <- t(matrix.with.summaries)
  
  #cat("Sum of deviances for simulated vs. observed single marginals:\n")
  #print(single.marginals.deviance.sum.obs.vs.sim.matrix)
  #cat("\n")

  # pretty printing ->
  display.vector <- c("s", rep("f", ncol(matrix.with.summaries)))

  # Because of transpose, we take [,1] which is rows instead of [,2] which is columns
  #colums.with.small.values <- unique(as.vector(which(matrix.with.summaries < 0.001 & matrix.with.summaries > 0, arr.ind = TRUE)[, 2]))  
  colums.with.small.values <- unique(as.vector(which(matrix.with.summaries < 0.001 & matrix.with.summaries > 0, arr.ind = TRUE)[, 1]))

  if (length(colums.with.small.values) > 0) {
    display.vector[colums.with.small.values + 1] <- "e"
  }

  if (columns.is.int) {
    display.vector <- c("s", rep("d", ncol(matrix.with.summaries)))
  } 
  # <- pretty printing

  tableOutput <- xtable(matrix.with.summaries,
        digits=3,
        display=display.vector,
        align=c("l", rep("r", ncol(matrix.with.summaries))),
        label=paste("tab:", identifier, sep=""), 
        caption=paste(matrix.description.long, " for ", true.model.name, " ", marginals.type, " marginals vs. ", compare.name, " ", marginals.type, " marginals for the ", method.name, " method specified in each row.", sep=""))
  filename <- paste("out/", method.name, "/", identifier, ".tex", sep="")
  print(tableOutput, file=filename)
  do.replacements(filename)

  # Include table in LaTeX automatically
  sink() # stop log
  sink(latex.filename, append = TRUE)
  cat("\\input{", identifier, "}\n\\clearpage\n", sep="")
  sink()
  sink(log.filename, append = TRUE) # start log again
}

