library(xtable)

source("include-unobserved.R")
source("start.R")

alpha <- 0.05
z.alpha <- qnorm(1 - alpha/2)

sink("log/unobserved.log")

cat("Using alpha =", alpha, "\n\n")

conf.int.output <- function(conf.int) {
  cat("An approximate", (1-alpha)*100, "% confidence interval for V is   [", round(conf.int[1], 3), ";", round(conf.int[2], 3), "]\n")
  cat("An approximate", (1-alpha)*100, "% confidence interval for 1-V is [", round(1-conf.int[2], 3), ";", round(1-conf.int[1], 3), "]\n")
  cat("Interval width is ", round(conf.int[2] - conf.int[1], 3), "\n")
}

unobserved.prob.matrix <- matrix(NA, nrow=length(dataset.names), 
  ncol=3, 
  dimnames=list(c(dataset.names), c("Estimate", "CV", "95% confidence interval")))

for (i in 1:length(dataset.names)) {
  dataset.name <- dataset.names[i]

  cat("Starting unobserved probability mass analysis for dataset \"", dataset.name, "\".\n", sep="")

  M1 <- get.singletons.count(dataset.name)
  M2 <- get.doubletons.count(dataset.name)
  N <- get.observation.count(dataset.name)

  V <- M1 / (N + 1)
  sigmasq <- V.sigmasq.estimate(N, M1, M2)  
  CV <- sqrt(sigmasq) / V
  V.conf.int.S <- V.confidence.interval(N, V, M1, M2, z.alpha, use.delta = FALSE)
  V.conf.int.T <- V.confidence.interval(N, V, M1, M2, z.alpha, use.delta = TRUE)

  unobserved.prob.matrix[i, 1] <- round(V, 3)
  unobserved.prob.matrix[i, 2] <- round(CV, 3)
  #unobserved.prob.matrix[i, 3] <- paste("[", round(V.conf.int.T[1], 3), ";", round(V.conf.int.T[2], 3), "]")
  unobserved.prob.matrix[i, 3] <- paste("[", round(V.conf.int.S[1], 3), ";", round(V.conf.int.S[2], 3), "]")

  cat("V (unobserved probability mass) =", round(V, 3), "\n")
  cat("1-V (observed probability mass) =", round(1-V, 3), "\n")  

  cat("CV (coefficient of variation)   = sigma / V =", round(CV, 3), "\n")

  cat("Using variance estimate and standard symmtric interval:\n")
  conf.int.output(V.conf.int.S)
  
  cat("Using variance estimate through the delta-method:\n")
  conf.int.output(V.conf.int.T)

  cat("Using the maximal variance:\n")
  var.max <- 1 / (N + 1)
  max.conf.int <- V + c(-1, 1) * z.alpha * sqrt(var.max)
  conf.int.output(max.conf.int)

  cat("Dataset \"", dataset.name, "\" done.\n\n", sep="")
}

sink()

tableOutput <- xtable(unobserved.prob.matrix,
      align=c("l", "r", "r", "r"),
      hline.after=seq(-1, length(dataset.names), 1),
      label=paste("tab:datasets-unobserved-prob-matrix", sep=""), 
      caption=paste("Table of estimates of unobserved probability mass. The coefficient of variation, CV, is calculated using estimates. The confidence interval is approximate and found using an asymptotic normality. Please refer to section ref{sec:Robbins} for further details.")) 
print(tableOutput, file=paste("out/unobserved-probability-mass.tex", sep=""))

